//hashmap of all subscribed commands and their handle callback
var subscribedCommands = {}; //each entry is a handle callback with the command as key

//called by a subscriber, to handle the given command
var brokerHandle = function(command, cb) {
    //cannot handle the same command multiple times
    if (subscribedCommands[command]) throw "Duplicate Command Handle: Cannot handle the same command twice.";
    
    //add the callback to this command
    subscribedCommands[command] = cb;
    
    //return broker so that handle can be chained
    return this;
};

//called by a subscriber, to remove the handle to a given command
var brokerRemoveHandle = function(command) {
    delete subscribedCommands[command];
    return this; //return broker so that rem handle can be chained
};

//called by publisher, to execute the given command
var brokerExecute = function(command) {
    //make sure that this command is being listened to; otherwise a registration order mistake has been made
    if (!subscribedCommands[command]) throw "Cannot execute given command as it does not exist yet.";
    
    //convert arguments obj to array and remove the command argument
    var args = Array.prototype.slice.call(arguments);
    args.shift();
    
    //call subscribed callback for this command with the given arguments
    subscribedCommands[command].apply(null, args);
    
    //return broker so that exec can be chained
    return this;
};

//published methods
exports.handle = brokerHandle;
exports.exec = brokerExecute;
exports.removeHandle = brokerRemoveHandle;